package ru.amse.javadependencies.zhukova.analyser;

import org.objectweb.asm.*;
import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.IPackage;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.DependencyGraph;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.ClassDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.TypeFeatureDependency;

import static ru.amse.javadependencies.zhukova.analyser.StaticAnaliseMethods.*;

import java.util.Set;
import java.util.HashSet;

/**
 * @author Anna R. Zhukova
 */

//todo generics from signatyre
class SpecialClassVisitor implements ClassVisitor {
    private IClass myClass;
    private final DependencyGraph<IClass> myClassDependencyGraph;
    private final DependencyGraph<IPackage> myPackageDependencyGraph;

    public SpecialClassVisitor(DependencyGraph<IClass> classDependencyGraph, DependencyGraph<IPackage> packageDependencyGraph) {
        myClassDependencyGraph = classDependencyGraph;
        myPackageDependencyGraph = packageDependencyGraph;
    }

    //Class Visitor
    public void visit(int version, int access, String name,
                      String signature, String superName, String[] interfaces) {
        // initializing client class
        if (myClass == null) {
            myClass = initializeClass(myClassDependencyGraph, myPackageDependencyGraph, name);
            if (myClass == null) {
                return;
            }
        }
        IPackage pckg = myClass.getPackage();
        myClassDependencyGraph.addClient(myClass);
        myPackageDependencyGraph.addClient(pckg);
        // extending dependency
        Set<IClass> superClasses = new HashSet<IClass>();
        IClass server = initializeClass(myClassDependencyGraph, myPackageDependencyGraph, superName);
        superClasses.add(server);
        for (String intrfc : interfaces) {
            IClass intr = initializeClass(myClassDependencyGraph, myPackageDependencyGraph, intrfc);
            superClasses.add(intr);
        }
        for (IClass cl : superClasses) {
            if (cl != null) {
                ClassDependency clDependency = new ClassDependency(cl);
                TypeFeatureDependency typeFeatureDep = new TypeFeatureDependency(cl, TypeFeatureDependency.Type.EXTENDING_DEPENDENCY);
                clDependency.addSubdependency(typeFeatureDep);
                myClass.addClassDependency(clDependency);
                pckg.addClassDependency(clDependency);
            }
        }
    }

    public void visitSource(String source, String debug) {
    }

    public void visitOuterClass(String owner, String name, String desc) {
    }

    public AnnotationVisitor visitAnnotation(String string, boolean b) {
        return null;
    }

    public void visitAttribute(Attribute attribute) {
    }

    public void visitInnerClass(String name, String outerName,
                                String innerName, int access) {
    }

    public FieldVisitor visitField(int access, String name, String desc,
                                   String signature, Object value) {
        if (myClass != null) {
            addTypeDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, desc, name, TypeFeatureDependency.Type.FIELD_DEPENDENCY);
        }
        return null;
    }

    public MethodVisitor visitMethod(int access, String name,
                                     String desc, String signature, String[] exceptions) {
        if (myClass == null) {
            return null;
        }
        String myMethodName = name + getArguments(desc);
        // arguments
        String newParametersDesc = desc.substring(desc.indexOf("(") + 1, desc.indexOf(")"));
        while (newParametersDesc.length() != 0) {
            int pos = newParametersDesc.indexOf(';');
            if (pos < 0) {
                break;
            }
            String parameter = newParametersDesc.substring(0, pos);
            newParametersDesc = newParametersDesc.substring(pos + 1, newParametersDesc.length());
            addTypeDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, parameter, myMethodName, TypeFeatureDependency.Type.METHOD_ARGUMENT_DEPENDENCY);
        }
        //return type
        String newReturnTypeDesc = desc.substring(desc.indexOf(")") + 1);
        addTypeDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, newReturnTypeDesc, myMethodName, TypeFeatureDependency.Type.METHOD_RETURN_TYPE_DEPENDENCY);
        return new SpecialMethodVisitor(myClass, myMethodName, myClassDependencyGraph, myPackageDependencyGraph);//this;
    }

    public void visitEnd() {
    }
}