package ru.amse.javadependencies.zhukova.modelbuilding;

import ru.amse.javadependencies.zhukova.ui.tree.CheckNode;
import ru.amse.javadependencies.zhukova.ui.tree.CheckRenderer;
import ru.amse.javadependencies.zhukova.ui.Main;
import ru.amse.javadependencies.zhukova.model.impl.Arc;

import javax.swing.*;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

/**
 * @author Anna R. Zhukova
 */
public class DependencyTreeBuilder {     

    public static JTree buildDependencyTree(final Main main) {
        CheckNode root = new CheckNode("Dependencies");
        CheckNode typeNode = new CheckNode("Type feature dependency");
        typeNode.add(new CheckNode("EXTENDING_DEPENDENCY"));
        typeNode.add(new CheckNode("FIELD_DEPENDENCY"));
        typeNode.add(new CheckNode("METHOD_ARGUMENT_DEPENDENCY"));
        typeNode.add(new CheckNode("METHOD_RETURN_TYPE_DEPENDENCY"));
        typeNode.add(new CheckNode("LOCAL_VARIABLE_DEPENDENCY"));
        root.add(typeNode);
        CheckNode elementNode = new CheckNode("Element feature dependency");
        root.add(elementNode);
        final JTree tree = new JTree(root);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setCellRenderer(new CheckRenderer());
        tree.addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {

                if (e.getButton() == MouseEvent.BUTTON1) {
                    TreePath treePath = tree.getPathForLocation(e.getX(), e.getY());
                    if (treePath != null) {
                        CheckNode node = (CheckNode) treePath.getLastPathComponent();
                        ArrayList<CheckNode> nodes = new ArrayList<CheckNode>();
                        boolean newIsSelected = !node.isSelected();
                        node.setSelected(newIsSelected, CheckNode.OUR_DIG_IN_SELECTION, nodes);
                        for (CheckNode checkNode : nodes) {
                            if (checkNode.isLeaf()) {
                                String name = checkNode.toString();
                                if (checkNode.getParent() != tree.getModel().getRoot()) {
                                    name = checkNode.getParent().toString() + ": " + name;
                                }
                                if (!newIsSelected) {
                                    Arc.getFilter().addBarrier(name);
                                } else {
                                    Arc.getFilter().removeBarrier(name);
                                }
                            }
                        }
                        ((DefaultTreeModel) tree.getModel()).nodeChanged(node);
                        tree.revalidate();
                        tree.repaint();
                        main.getGraphPane().repaint();
                        main.getDependencyTreePanel().repaint();
                    }
                }
            }
        });
        return tree;
    }
}
